@@ -4,8 +4,9 @@ from typing import Any, Dict
 
 import pandas as pd
 from scipy.stats import norm
+from slpp import slpp as lua
 
-from pricer import config as cfg, utils
+from pricer import config as cfg, io, utils
 
 logger = logging.getLogger(__name__)
 
@@ -14,9 +15,7 @@ def analyse_buy_policy(MAX_BUY_STD: int = 2) -> None:
     """Create buy policy."""
     logger.debug(f"max buy std {MAX_BUY_STD}")
 
-    path = "data/intermediate/item_table.parquet"
-    logger.debug(f"Reading item_table parquet from {path}")
-    item_table = pd.read_parquet(path)
+    item_table = io.reader("intermediate", "item_table", "parquet")
 
     buy_policy = item_table[item_table["Buy"] == True]
     subset_cols = [
@@ -29,9 +28,8 @@ def analyse_buy_policy(MAX_BUY_STD: int = 2) -> None:
     ]
     buy_policy = buy_policy[subset_cols]
 
-    path = "data/intermediate/listing_each.parquet"
-    logger.debug(f"Reading listing_each parquet from {path}")
-    listing_each = pd.read_parquet(path)
+    listing_each = io.reader("intermediate", "listing_each", "parquet")
+
     listing_each = listing_each.sort_values("price_per")
 
     rank_list = listing_each.join(buy_policy, on="item").dropna()
@@ -49,20 +47,14 @@ def analyse_buy_policy(MAX_BUY_STD: int = 2) -> None:
     )
 
     rank_list = rank_list[rank_list["updated_replenish_z"] > rank_list["pred_z"]]
-
-    path = "data/outputs/buy_rank.parquet"
-    logger.debug(f"Writing buy_rank parquet to {path}")
-    rank_list.to_parquet(path, compression="gzip")
+    io.writer(rank_list, "reporting", "buy_rank", "parquet")
 
     buy_policy["buy_price"] = rank_list.groupby("item")["price_per"].max()
     buy_policy["buy_price"] = buy_policy["buy_price"].fillna(1).astype(int)
 
     buy_policy.index.name = "item"
     buy_policy = buy_policy.reset_index()
-
-    path = "data/outputs/buy_policy.parquet"
-    logger.debug(f"Writing buy_policy parquet to {path}")
-    buy_policy.to_parquet(path, compression="gzip")
+    io.writer(buy_policy, "outputs", "buy_policy", "parquet")
 
 
 def encode_buy_campaign(buy_policy: pd.DataFrame) -> Dict[str, Dict[str, Any]]:
@@ -88,9 +80,7 @@ def encode_buy_campaign(buy_policy: pd.DataFrame) -> Dict[str, Dict[str, Any]]:
 
 def write_buy_policy() -> None:
     """Writes the buy policy to all accounts."""
-    path = "data/outputs/buy_policy.parquet"
-    logger.debug(f"Reading buy_policy parquet from {path}")
-    buy_policy = pd.read_parquet(path)
+    buy_policy = io.reader("outputs", "buy_policy", "parquet")
 
     cols = ["item", "buy_price"]
     new_snatch = encode_buy_campaign(buy_policy[cols])
@@ -99,14 +89,14 @@ def write_buy_policy() -> None:
 
     for account in cfg.us.get("accounts", []):
         path = utils.make_lua_path(account_name=account, datasource="Auc-Advanced")
-        data = utils.read_lua(path)
+        data = io.reader(name=path, ftype="lua")
         snatch = data["AucAdvancedData"]["UtilSearchUiData"]["Current"]
         snatch["snatch.itemsList"] = {}
         snatch = snatch["snatch.itemsList"]
         data["AucAdvancedData"]["UtilSearchUiData"]["Current"][
             "snatch.itemsList"
         ] = new_snatch
-        utils.write_lua(data, path)
+        io.writer(data, name=path, ftype="lua")
 
 
 def encode_sell_campaign(sell_policy: pd.DataFrame) -> Dict[str, Any]:
@@ -148,9 +138,7 @@ def encode_sell_campaign(sell_policy: pd.DataFrame) -> Dict[str, Any]:
 
 def write_sell_policy() -> None:
     """Writes the sell policy to accounts."""
-    path = "data/outputs/sell_policy.parquet"
-    logger.debug(f"Reading sell_policy parquet from {path}")
-    sell_policy = pd.read_parquet(path)
+    sell_policy = io.reader("outputs", "sell_policy", "parquet")
 
     cols = ["item", "proposed_buy", "proposed_bid", "sell_count", "stack", "duration"]
     new_appraiser = encode_sell_campaign(sell_policy[cols])
@@ -158,11 +146,11 @@ def write_sell_policy() -> None:
     # Read client lua, replace with
     for account in cfg.us.get("accounts", []):
         path = utils.make_lua_path(account_name=account, datasource="Auc-Advanced")
-        data = utils.read_lua(path)
+        data = io.reader(name=path, ftype="lua")
         data["AucAdvancedConfig"]["profile.Default"]["util"][
             "appraiser"
         ] = new_appraiser
-        utils.write_lua(data, path)
+        io.writer(data, name=path, ftype="lua")
 
 
 def analyse_sell_policy(
@@ -174,17 +162,11 @@ def analyse_sell_policy(
     MIN_PROFIT_PCT: float = 0.015,
 ) -> None:
     """Creates sell policy based on information."""
-    path = "data/intermediate/item_table.parquet"
-    logger.debug(f"Reading item_table parquet from {path}")
-    item_table = pd.read_parquet(path)
-
-    path = "data/intermediate/listing_each.parquet"
-    logger.debug(f"Reading listing_each parquet from {path}")
-    listing_each = pd.read_parquet(path)
-
-    path = "data/intermediate/item_volume_change_probability.parquet"
-    logger.debug(f"Reading item_volume_change_probability parquet from {path}")
-    item_volume_change_probability = pd.read_parquet(path)
+    item_table = io.reader("intermediate", "item_table", "parquet")
+    listing_each = io.reader("intermediate", "listing_each", "parquet")
+    item_volume_change_probability = io.reader(
+        "intermediate", "item_volume_change_probability", "parquet"
+    )
 
     cols = [
         "deposit",
@@ -278,13 +260,131 @@ def analyse_sell_policy(
         adjust_stack, "min_sell"
     ]
 
-    path = "data/outputs/sell_policy.parquet"
-    logger.debug(f"Writing sell_policy parquet to {path}")
-    sell_policy.to_parquet(path, compression="gzip")
+    io.writer(sell_policy, "outputs", "sell_policy", "parquet")
 
     listing_profits = listing_profits.set_index(["rank", "item"])[
         "estimated_profit"
     ].unstack()
-    path = "data/reporting/listing_profits.parquet"
-    logger.debug(f"Writing sell_policy parquet to {path}")
-    listing_profits.to_parquet(path, compression="gzip")
+    io.writer(listing_profits, "reporting", "listing_profits", "parquet")
+
+
+def analyse_make_policy() -> None:
+    """Prints what potions to make."""
+    item_table = io.reader("intermediate", "item_table", "parquet")
+    item_table.index.name = "item"
+
+    cols = [
+        "item_id",
+        "made_from",
+        "make_pass",
+        "inv_total_all",
+        "mean_holding",
+        "inv_ahm_bag",
+        "inv_ahm_bank",
+        "Sell",
+    ]
+    make = item_table[cols]
+
+    user_items = cfg.ui.copy()
+
+    make["make_ideal"] = make["mean_holding"] - make["inv_total_all"]
+    make["make_counter"] = make["make_ideal"].apply(lambda x: max(x, 0))
+    make["make_mat_available"] = make["inv_ahm_bag"] + make["inv_ahm_bank"]
+    make["make_actual"] = 0
+    make["make_mat_flag"] = 0
+
+    # Iterates through the table one at a time, to ensure fair distribution of mat usage
+    # Tests if reached counter and is made from stuff
+    # Checks the material count can go down first before decrementing
+    # If after each check, append to list to see for any changes on any pass through
+    change = [True]
+    while any(change):
+        change = []
+
+        for item, row in make.iterrows():
+
+            made_from = user_items[item].get("made_from", {})
+            under_counter = row["make_actual"] < row["make_counter"]
+            make_pass = row["make_pass"]
+
+            if made_from and under_counter and not (make_pass):
+                item_increment = True
+                for material, qty in made_from.items():
+                    if "Vial" not in material:
+                        item_increment = (
+                            make.loc[material, "make_mat_available"] >= qty
+                        ) & item_increment
+
+                if item_increment:
+                    for material, qty in user_items[item].get("made_from", {}).items():
+                        make.loc[material, "make_mat_available"] -= qty
+                        make.loc[material, "make_mat_flag"] = 1
+                    make.loc[item, "make_actual"] += 1
+
+                change.append(item_increment)
+
+    make_me = make[(make["make_pass"] == 0) & (make["make_actual"] > 0)]["make_actual"]
+    make_me.name = "Automake"
+    print(make_me)
+    make_me = make_me.to_dict()
+
+    make_main = make[(make["make_pass"] == 1) & (make["make_ideal"] > 0)]["make_ideal"]
+    make_main.name = "Make on main"
+    print(make_main)
+
+    make_should = make[
+        (
+            (make["make_pass"] == 0)
+            & (make["make_ideal"] > make["make_actual"])
+            & (make["made_from"] == 1)
+        )
+    ]["make_ideal"]
+    make_should.name = "Missing mats"
+    print(make_should)
+
+    materials_list = make[make["make_mat_flag"] == 1]["item_id"].to_dict()
+    sell_items = make[make["Sell"] == 1]["item_id"]
+
+    path = utils.make_lua_path(
+        account_name="396255466#1", datasource="TradeSkillMaster"
+    )
+    content = io.reader(name=path, ftype="lua", custom="rb")
+
+    start, end = utils.find_attribute_location(
+        content, b'["f@Alliance - Grobbulus@internalData@crafts"]'
+    )
+
+    crafting = content[start:end]
+    crafting_dict = lua.decode("{" + crafting.decode("ascii") + "}")
+
+    for _, item_data in crafting_dict[
+        "f@Alliance - Grobbulus@internalData@crafts"
+    ].items():
+        queued = make_me.get(item_data.get("name"), 0)
+
+        if "queued" in item_data:
+            item_data["queued"] = queued
+
+    new_crafting = utils.dict_to_lua(crafting_dict).encode("ascii")
+    new_crafting = new_crafting.replace(
+        b"\nf@Alliance - Grobbulus@internalData@crafts",
+        b'\n["f@Alliance - Grobbulus@internalData@crafts"]',
+    )
+
+    content = content[:start] + new_crafting + content[end:]
+
+    start, end = utils.find_attribute_location(content, b'["p@Default@userData@items"]')
+
+    item_text = '["p@Default@userData@items"] = {'
+    for _, item_code in materials_list.items():
+        item_text += f'["i:{item_code}"] = "Herbs", '
+
+    for item_name, item_code in sell_items.items():
+        if item_name not in materials_list:
+            item_text += f'["i:{item_code}"] = "Sell", '
+
+    item_text += "}"
+
+    content = content[:start] + item_text.encode("ascii") + content[end:]
+    io.writer(content, name=path, ftype="lua", custom="wb")
+    io.writer(make, "outputs", "make", "parquet")
